@@ -343,16 +343,16 @@ Command help:
         print _('Back ends:'), ', '.join(backends)
 
     def do_install(self, tracker_home, args):
-        ""'''Usage: install [template [backend [admin password [key=val[,key=val]]]]]
+        ""'''Usage: install [template [backend [key=val[,key=val]]]]
         Install a new Roundup tracker.
 
         The command will prompt for the tracker home directory
         (if not supplied through TRACKER_HOME or the -i option).
-        The template, backend and admin password may be specified
-        on the command-line as arguments, in that order.
+        The template and backend may be specified on the command-line
+        as arguments, in that order.
 
-        The last command line argument allows to pass initial values
-        for config options.  For example, passing
+        Command line arguments following the backend allows you to
+        pass initial values for config options.  For example, passing
         "web_http_auth=no,rdbms_user=dinsdale" will override defaults
         for options http_auth in section [web] and user in section [rdbms].
         Please be careful to not use spaces in this argument! (Enclose
@@ -1054,11 +1054,14 @@ Erase it? Y/N: """))
                 raise UsageError, _('no such %(classname)s node "%(nodeid)s"')%locals()
         return 0
 
-    def do_export(self, args):
-        ""'''Usage: export [class[,class]] export_dir
+    def do_export(self, args, export_files=True):
+        ""'''Usage: export [[-]class[,class]] export_dir
         Export the database to colon-separated-value files.
+        To exclude the files (e.g. for the msg or file class),
+        use the exporttables command.
 
-        Optionally limit the export to just the names classes.
+        Optionally limit the export to just the named classes
+        or exclude the named classes, if the 1st argument starts with '-'.
 
         This action exports the current data from the database into
         colon-separated-value files that are placed in the nominated
@@ -1072,7 +1075,11 @@ Erase it? Y/N: """))
 
         # get the list of classes to export
         if len(args) == 2:
-            classes = args[0].split(',')
+            if args[0].startswith('-'):
+                classes = [ c for c in self.db.classes.keys()
+                            if not c in args[0][1:].split(',') ]
+            else:
+                classes = args[0].split(',')
         else:
             classes = self.db.classes.keys()
 
@@ -1087,6 +1094,10 @@ Erase it? Y/N: """))
         for classname in classes:
             cl = self.get_class(classname)
 
+            if not export_files and hasattr(cl, 'export_files'):
+                sys.stdout.write('Exporting %s WITHOUT the files\r\n'%
+                    classname)
+
             f = open(os.path.join(dir, classname+'.csv'), 'wb')
             writer = csv.writer(f, colon_separated)
 
@@ -1098,8 +1109,14 @@ Erase it? Y/N: """))
 
             # all nodes for this class
             for nodeid in cl.getnodeids():
+                if self.verbose:
+                    sys.stdout.write('Exporting %s - %s\r'%(classname, nodeid))
+                    sys.stdout.flush()
+                if self.verbose:
+                    sys.stdout.write('Exporting %s - %s\r'%(classname, nodeid))
+                    sys.stdout.flush()
                 writer.writerow(cl.export_list(propnames, nodeid))
-                if hasattr(cl, 'export_files'):
+                if export_files and hasattr(cl, 'export_files'):
                     cl.export_files(dir, nodeid)
 
             # close this file
@@ -1107,11 +1124,29 @@ Erase it? Y/N: """))
 
             # export the journals
             jf = open(os.path.join(dir, classname+'-journals.csv'), 'wb')
+            if self.verbose:
+                sys.stdout.write("\nExporting Journal for %s\n" % classname)
+                sys.stdout.flush()
             journals = csv.writer(jf, colon_separated)
             map(journals.writerow, cl.export_journals())
             jf.close()
         return 0
 
+    def do_exporttables(self, args):
+        ""'''Usage: exporttables [[-]class[,class]] export_dir
+        Export the database to colon-separated-value files, excluding the
+        files below $TRACKER_HOME/db/files/ (which can be archived separately).
+        To include the files, use the export command.
+
+        Optionally limit the export to just the named classes
+        or exclude the named classes, if the 1st argument starts with '-'.
+
+        This action exports the current data from the database into
+        colon-separated-value files that are placed in the nominated
+        destination directory.
+        '''
+        return self.do_export(args, export_files=False)
+
     def do_import(self, args):
         ""'''Usage: import import_dir
         Import a database from the directory containing CSV files,
@@ -1162,14 +1197,15 @@ Erase it? Y/N: """))
                     file_props = r
                     continue
 
-                sys.stdout.write('Importing %s - %d\r'%(classname, n))
-                sys.stdout.flush()
+                if self.verbose:
+                    sys.stdout.write('Importing %s - %s\r'%(classname, n))
+                    sys.stdout.flush()
 
                 # do the import and figure the current highest nodeid
-                nodeid = int(cl.import_list(file_props, r))
+                nodeid = cl.import_list(file_props, r)
                 if hasattr(cl, 'import_files'):
                     cl.import_files(dir, nodeid)
-                maxid = max(maxid, nodeid)
+                maxid = max(maxid, int(nodeid))
             print
             f.close()
 
@@ -1399,7 +1435,7 @@ Erase it? Y/N: """))
 
     def main(self):
         try:
-            opts, args = getopt.getopt(sys.argv[1:], 'i:u:hcdsS:v')
+            opts, args = getopt.getopt(sys.argv[1:], 'i:u:hcdsS:vV')
         except getopt.GetoptError, e:
             self.usage(str(e))
             return 1
