@@ -28,9 +28,10 @@ import sys
 # module name, have_backend quietly returns False.
 # Otherwise the error is reraised.
 _modules = {
-    'mysql': 'MySQLdb',
-    'postgresql': 'psycopg',
-    'tsearch2': 'psycopg',
+    'mysql': ('MySQLdb',),
+    'postgresql': ('psycopg',),
+    'tsearch2': ('psycopg',),
+    'sqlite': ('pysqlite', 'pysqlite2', 'sqlite3'),
 }
 
 def get_backend(name):
@@ -65,10 +66,10 @@ def have_backend(name):
         get_backend(name)
         return 1
     except ImportError, e:
-        global _modules
-        if not str(e).startswith('No module named %s'
-                % _modules.get(name, name)):
-            raise
+        for name in _modules.get(name, (name,)):
+            if str(e).startswith('No module named %s'%name):
+                return 0
+        raise
     return 0
 
 def list_backends():
