@@ -42,11 +42,12 @@ __docformat__ = 'restructuredtext'
 BACKWARDS_COMPATIBLE = 1
 
 from roundup import hyperdb, date, password, roundupdb, security
+from roundup.support import reversed
 import logging
 import metakit
 from sessions_dbm import Sessions, OneTimeKeys
 import re, marshal, os, sys, time, calendar, shutil
-from indexer_common import Indexer
+from indexer_common import Indexer as CommonIndexer
 import locking
 from roundup.date import Range
 from blobfiles import files_in_dir
@@ -146,8 +147,18 @@ class _Database(hyperdb.Database, roundupdb.Database):
     # --- end of ping's spec
 
     # --- exposed methods
-    def commit(self):
-        '''commit all changes to the database'''
+    def commit(self, fail_ok=False):
+        ''' Commit the current transactions.
+
+        Save all data changed since the database was opened or since the
+        last commit() or rollback().
+
+        fail_ok indicates that the commit is allowed to fail. This is used
+        in the web interface when committing cleaning of the session
+        database. We don't care if there's a concurrency issue there.
+
+        The only backend this seems to affect is postgres.
+        '''
         if self.dirty:
             self._db.commit()
             for cl in self.classes.values():
@@ -1155,8 +1166,7 @@ class Class(hyperdb.Class):
         self.db.commit()
     # ---- end of ping's spec
 
-    def filter(self, search_matches, filterspec, sort=(None,None),
-            group=(None,None)):
+    def _filter(self, search_matches, filterspec, proptree):
         '''Return a list of the ids of the active nodes in this class that
         match the 'filter' spec, sorted by the group spec and then the
         sort spec
@@ -1326,37 +1336,58 @@ class Class(hyperdb.Class):
             iv = v.filter(ff)
             v = v.remapwith(iv)
 
-        if sort or group:
-            sortspec = []
-            rev = []
-            for dir, propname in group, sort:
-                if propname is None: continue
-                isreversed = 0
-                if dir == '-':
-                    isreversed = 1
-                try:
-                    prop = getattr(v, propname)
-                except AttributeError:
-                    logging.getLogger("hyperdb").error(
-                        "MK has no property %s" % propname)
-                    continue
-                propclass = self.ruprops.get(propname, None)
+        # Handle all the sorting we can inside Metakit. If we encounter
+        # transitive attributes or a Multilink on the way, we sort by
+        # what we have so far and defer the rest to the outer sorting
+        # routine. We mark the attributes for which sorting has been
+        # done with sort_done. Of course the whole thing works only if
+        # we do it backwards.
+        sortspec = []
+        rev = []
+        sa = []
+        if proptree:
+            sa = reversed(proptree.sortattr)
+        for pt in sa:
+            if pt.parent != proptree:
+                break;
+            propname = pt.name
+            dir = pt.sort_direction
+            assert (dir and propname)
+            isreversed = 0
+            if dir == '-':
+                isreversed = 1
+            try:
+                prop = getattr(v, propname)
+            except AttributeError:
+                logging.getLogger("hyperdb").error(
+                    "MK has no property %s" % propname)
+                continue
+            propclass = self.ruprops.get(propname, None)
+            if propclass is None:
+                propclass = self.privateprops.get(propname, None)
                 if propclass is None:
-                    propclass = self.privateprops.get(propname, None)
-                    if propclass is None:
-                        logging.getLogger("hyperdb").error(
-                            "Schema has no property %s" % propname)
-                        continue
-                if isinstance(propclass, hyperdb.Link):
-                    linkclass = self.db.getclass(propclass.classname)
-                    lv = linkclass.getview()
-                    lv = lv.rename('id', propname)
-                    v = v.join(lv, prop, 1)
-                    prop = getattr(v, linkclass.orderprop())
-                if isreversed:
-                    rev.append(prop)
-                sortspec.append(prop)
-            v = v.sortrev(sortspec, rev)[:] #XXX Metakit bug
+                    logging.getLogger("hyperdb").error(
+                        "Schema has no property %s" % propname)
+                    continue
+            # Dead code: We dont't find Links here (in sortattr we would
+            # see the order property of the link, but this is not in the
+            # first level of the tree). The code is left in because one
+            # day we might want to properly implement this.  The code is
+            # broken because natural-joining to the Link-class can
+            # produce name-clashes wich result in broken sorting.
+            if isinstance(propclass, hyperdb.Link):
+                linkclass = self.db.getclass(propclass.classname)
+                lv = linkclass.getview()
+                lv = lv.rename('id', propname)
+                v = v.join(lv, prop, 1)
+                prop = getattr(v, linkclass.orderprop())
+            if isreversed:
+                rev.append(prop)
+            sortspec.append(prop)
+            pt.sort_done = True
+        sortspec.reverse()
+        rev.reverse()
+        v = v.sortrev(sortspec, rev)[:] #XXX Metakit bug
 
         rslt = []
         for row in v:
@@ -1819,6 +1850,9 @@ class FileClass(hyperdb.FileClass, Class):
             os.makedirs(os.path.dirname(dest))
         shutil.copyfile(source, dest)
 
+        if self.properties['content'].indexme:
+            return
+
         mime_type = None
         if self.getprops().has_key('type'):
             mime_type = self.get(nodeid, 'type')
@@ -1875,6 +1909,9 @@ class FileClass(hyperdb.FileClass, Class):
         f.write(content)
         f.close()
 
+        if not self.properties['content'].indexme:
+            return newid
+
         mimetype = propvalues.get('type', self.default_mime_type)
         self.db.indexer.add_text((self.classname, newid, 'content'), content,
             mimetype)
@@ -1912,7 +1949,7 @@ class FileClass(hyperdb.FileClass, Class):
             f.close()
 
             if self.properties['content'].indexme:
-                mimetype = self.get('type', self.default_mime_type)
+                mimetype = propvalues.get('type', self.default_mime_type)
                 self.db.indexer.add_text((self.classname, itemid, 'content'),
                     content, mimetype)
 
@@ -1961,8 +1998,9 @@ class IssueClass(Class, roundupdb.IssueClass):
 
 CURVERSION = 2
 
-class MetakitIndexer(Indexer):
+class MetakitIndexer(CommonIndexer):
     def __init__(self, db):
+        CommonIndexer.__init__(self, db)
         self.path = os.path.join(db.config.DATABASE, 'index.mk4')
         self.db = metakit.storage(self.path, 1)
         self.datadb = db._db
