@@ -10,7 +10,12 @@
 __docformat__ = 'restructuredtext'
 
 import os, shutil, popen2, time
-import psycopg
+try:
+    import psycopg
+    from psycopg import QuotedString
+except:
+    from psycopg2 import psycopg1 as psycopg
+    from psycopg2.extensions import QuotedString
 import logging
 
 from roundup import hyperdb, date
@@ -65,6 +70,8 @@ def db_command(config, command):
 def pg_command(cursor, command):
     '''Execute the postgresql command, which may be blocked by some other
     user connecting to the database, and return a true value if it succeeds.
+    
+    If there is a concurrent update, retry the command.
     '''
     try:
         cursor.execute(command)
@@ -73,10 +80,18 @@ def pg_command(cursor, command):
         if response.find('FATAL') != -1:
             raise RuntimeError, response
         elif response.find('ERROR') != -1:
-            if response.find('is being accessed by other users') == -1:
-                raise RuntimeError, response
-            time.sleep(1)
-            return 0
+            msgs = [
+                'is being accessed by other users',
+                'could not serialize access due to concurrent update',
+            ]
+            can_retry = 0
+            for msg in msgs:
+                if response.find(msg) == -1:
+                    can_retry = 1
+            if can_retry:
+                time.sleep(1)
+                return 0
+            raise RuntimeError, response
     return 1
 
 def db_exists(config):
@@ -178,10 +193,28 @@ class Database(rdbms_common.Database):
     def __repr__(self):
         return '<roundpsycopgsql 0x%x>' % id(self)
 
+    def sql_commit(self, fail_ok=False):
+        ''' Actually commit to the database.
+        '''
+        logging.getLogger('hyperdb').info('commit')
+
+        try:
+            self.conn.commit()
+        except psycopg.ProgrammingError, message:
+            # we've been instructed that this commit is allowed to fail
+            if fail_ok and str(message).endswith('could not serialize '
+                    'access due to concurrent update'):
+                logging.getLogger('hyperdb').info('commit FAILED, but fail_ok')
+            else:
+                raise
+
+        # open a new cursor for subsequent work
+        self.cursor = self.conn.cursor()
+
     def sql_stringquote(self, value):
         ''' psycopg.QuotedString returns a "buffer" object with the
             single-quotes around it... '''
-        return str(psycopg.QuotedString(str(value)))[1:-1]
+        return str(QuotedString(str(value)))[1:-1]
 
     def sql_index_exists(self, table_name, index_name):
         sql = 'select count(*) from pg_indexes where ' \
@@ -220,11 +253,13 @@ class Database(rdbms_common.Database):
             self.cursor.execute('DROP SEQUENCE _%s_ids'%cn)
             self.cursor.execute('CREATE SEQUENCE _%s_ids'%cn)
 
+class PostgresqlClass:
+    order_by_null_values = '(%s is not NULL)'
 
-class Class(rdbms_common.Class):
+class Class(PostgresqlClass, rdbms_common.Class):
     pass
-class IssueClass(rdbms_common.IssueClass):
+class IssueClass(PostgresqlClass, rdbms_common.IssueClass):
     pass
-class FileClass(rdbms_common.FileClass):
+class FileClass(PostgresqlClass, rdbms_common.FileClass):
     pass
 
