@@ -285,4 +285,8 @@ class Indexer(IndexerBase):
         ''' load last saved index info. '''
         self.load_index(reload=1)
 
+    def close(self):
+        pass
+
+
 # vim: set filetype=python ts=4 sw=4 et si
