@@ -221,9 +221,10 @@ class SearchAction(Action):
 
     def fakeFilterVars(self):
         """Add a faked :filter form variable for each filtering prop."""
-        props = self.db.classes[self.classname].getprops()
+        cls = self.db.classes[self.classname]
         for key in self.form.keys():
-            if not props.has_key(key):
+            prop = cls.get_transitive_prop(key)
+            if not prop:
                 continue
             if isinstance(self.form[key], type([])):
                 # search for at least one entry which is not empty
@@ -235,7 +236,7 @@ class SearchAction(Action):
             else:
                 if not self.form[key].value:
                     continue
-                if isinstance(props[key], hyperdb.String):
+                if isinstance(prop, hyperdb.String):
                     v = self.form[key].value
                     l = token.token_split(v)
                     if len(l) > 1 or l[0] != v:
@@ -627,7 +628,7 @@ class PassResetAction(Action):
         if self.form.has_key('otk'):
             # pull the rego information out of the otk database
             otk = self.form['otk'].value
-            uid = otks.get(otk, 'uid')
+            uid = otks.get(otk, 'uid', default=None)
             if uid is None:
                 self.client.error_message.append(
                     self._("Invalid One Time Key!\n"
