@@ -6,7 +6,7 @@ __docformat__ = 'restructuredtext'
 
 import base64, binascii, cgi, codecs, mimetypes, os
 import random, re, rfc822, stat, time, urllib, urlparse
-import Cookie
+import Cookie, socket, errno
 
 from roundup import roundupdb, date, hyperdb, password
 from roundup.cgi import templating, cgitb, TranslationService
@@ -334,7 +334,7 @@ class Client:
         sessions.clean(now)
         self.db.getOTKManager().clean(now)
         sessions.set('last_clean', last_use=time.time())
-        self.db.commit()
+        self.db.commit(fail_ok=True)
 
     def determine_charset(self):
         """Look for client charset in the form parameters or browser cookie.
@@ -821,7 +821,12 @@ class Client:
         if not self.headers_done:
             self.header()
         if self.env['REQUEST_METHOD'] != 'HEAD':
-            self.request.wfile.write(content)
+            try:
+                self.request.wfile.write(content)
+            except socket.error, error:
+                # the end-user has gone away
+                if error.errno != errno.EPIPE:
+                    raise
 
     def write_html(self, content):
         if not self.headers_done:
@@ -840,7 +845,12 @@ class Client:
             content = content.encode(self.charset, 'xmlcharrefreplace')
 
         # and write
-        self.request.wfile.write(content)
+        try:
+            self.request.wfile.write(content)
+        except socket.error, error:
+            # the end-user has gone away
+            if error.errno != errno.EPIPE:
+                raise
 
     def setHeader(self, header, value):
         '''Override a header to be returned to the user's browser.
