@@ -1,3 +1,5 @@
+from __future__ import nested_scopes
+
 """Implements the API used in the HTML templating for the web interface.
 """
 
@@ -17,10 +19,9 @@ todo = '''
 
 __docformat__ = 'restructuredtext'
 
-from __future__ import nested_scopes
 
 import sys, cgi, urllib, os, re, os.path, time, errno, mimetypes, csv
-import calendar
+import calendar, textwrap
 
 from roundup import hyperdb, date, support
 from roundup import i18n
@@ -392,12 +393,28 @@ def lookupKeys(linkcl, key, ids, num_re=re.compile('^-?\d+$')):
             l.append(entry)
     return l
 
+def _set_input_default_args(dic):
+    # 'text' is the default value anyway --
+    # but for CSS usage it should be present
+    dic.setdefault('type', 'text')
+    # useful e.g for HTML LABELs:
+    if not dic.has_key('id'):
+        try:
+            if dic['text'] in ('radio', 'checkbox'):
+                dic['id'] = '%(name)s-%(value)s' % dic
+            else:
+                dic['id'] = dic['name']
+        except KeyError:
+            pass
+
 def input_html4(**attrs):
     """Generate an 'input' (html4) element with given attributes"""
+    _set_input_default_args(attrs) 
     return '<input %s>'%' '.join(['%s="%s"'%item for item in attrs.items()])
 
 def input_xhtml(**attrs):
     """Generate an 'input' (xhtml) element with given attributes"""
+    _set_input_default_args(attrs)
     return '<input %s/>'%' '.join(['%s="%s"'%item for item in attrs.items()])
 
 class HTMLInputMixin:
@@ -604,8 +621,7 @@ class HTMLClass(HTMLInputMixin, HTMLPermissions):
         idlessprops.sort()
         return ['id'] + idlessprops
 
-    def filter(self, request=None, filterspec={}, sort=(None,None),
-            group=(None,None)):
+    def filter(self, request=None, filterspec={}, sort=[], group=[]):
         ''' Return a list of items from this class, filtered and sorted
             by the current requested filterspec/filter/sort/group args
 
@@ -769,27 +785,42 @@ class _HTMLItem(HTMLInputMixin, HTMLPermissions):
 
     def __getitem__(self, item):
         ''' return an HTMLProperty instance
+            this now can handle transitive lookups where item is of the
+            form x.y.z
         '''
         #print 'HTMLItem.getitem', (self, item)
         if item == 'id':
             return self._nodeid
 
+        items = item.split('.', 1)
+        has_rest = len(items) > 1
+
         # get the property
-        prop = self._props[item]
+        prop = self._props[items[0]]
+
+        if has_rest and not isinstance(prop, (hyperdb.Link, hyperdb.Multilink)):
+            raise KeyError, item
 
         # get the value, handling missing values
         value = None
         if int(self._nodeid) > 0:
-            value = self._klass.get(self._nodeid, item, None)
+            value = self._klass.get(self._nodeid, items[0], None)
         if value is None:
-            if isinstance(self._props[item], hyperdb.Multilink):
+            if isinstance(prop, hyperdb.Multilink):
                 value = []
 
         # look up the correct HTMLProperty class
+        htmlprop = None
         for klass, htmlklass in propclasses:
             if isinstance(prop, klass):
-                return htmlklass(self._client, self._classname,
-                    self._nodeid, prop, item, value, self._anonymous)
+                htmlprop = htmlklass(self._client, self._classname,
+                    self._nodeid, prop, items[0], value, self._anonymous)
+        if htmlprop is not None:
+            if has_rest:
+                if isinstance(htmlprop, MultilinkHTMLProperty):
+                    return [h[items[1]] for h in htmlprop]
+                return htmlprop[items[1]]
+            return htmlprop
 
         raise KeyError, item
 
@@ -1248,6 +1279,34 @@ class StringHTMLProperty(HTMLProperty):
             s = self.hyper_re.sub(self._hyper_repl, s)
         return s
 
+    def wrapped(self, escape=1, hyperlink=1):
+        '''Render a "wrapped" representation of the property.
+
+        We wrap long lines at 80 columns on the nearest whitespace. Lines
+        with no whitespace are not broken to force wrapping.
+
+        Note that unlike plain() we default wrapped() to have the escaping
+        and hyperlinking turned on since that's the most common usage.
+
+        - "escape" turns on/off HTML quoting
+        - "hyperlink" turns on/off in-text hyperlinking of URLs, email
+          addresses and designators
+        '''
+        if not self.is_view_ok():
+            return self._('[hidden]')
+
+        if self._value is None:
+            return ''
+        s = support.wrap(str(self._value), width=80)
+        if escape:
+            s = cgi.escape(s)
+        if hyperlink:
+            # no, we *must* escape this text
+            if not escape:
+                s = cgi.escape(s)
+            s = self.hyper_re.sub(self._hyper_repl, s)
+        return s
+
     def stext(self, escape=0, hyperlink=1):
         ''' Render the value of the property as StructuredText.
 
@@ -1293,9 +1352,11 @@ class StringHTMLProperty(HTMLProperty):
         else:
             value = cgi.escape(str(self._value))
 
-        value = '&quot;'.join(value.split('"'))
-        return '<textarea name="%s" rows="%s" cols="%s">%s</textarea>'%(
-            self._formname, rows, cols, value)
+            value = '&quot;'.join(value.split('"'))
+        name = self._formname
+        return ('<textarea name="%(name)s" id="%(name)s"'
+                ' rows="%(rows)s" cols="%(cols)s">'
+                 '%(value)s</textarea>') % locals()
 
     def email(self, escape=1):
         ''' Render the value of the property as an obscured email address
@@ -1330,7 +1391,7 @@ class PasswordHTMLProperty(HTMLProperty):
             return ''
         return self._('*encrypted*')
 
-    def field(self, size = 30):
+    def field(self, size=30):
         ''' Render a form edit field for the property.
 
             If not editable, just display the value via plain().
@@ -1340,7 +1401,7 @@ class PasswordHTMLProperty(HTMLProperty):
 
         return self.input(type="password", name=self._formname, size=size)
 
-    def confirm(self, size = 30):
+    def confirm(self, size=30):
         ''' Render a second form edit field for the property, used for
             confirmation that the user typed the password correctly. Generates
             a field with name "@confirm@name".
@@ -1351,7 +1412,9 @@ class PasswordHTMLProperty(HTMLProperty):
             return ''
 
         return self.input(type="password",
-            name="@confirm@%s"%self._formname, size=size)
+            name="@confirm@%s"%self._formname,
+            id="%s-confirm"%self._formname,
+            size=size)
 
 class NumberHTMLProperty(HTMLProperty):
     def plain(self):
@@ -1365,7 +1428,7 @@ class NumberHTMLProperty(HTMLProperty):
 
         return str(self._value)
 
-    def field(self, size = 30):
+    def field(self, size=30):
         ''' Render a form edit field for the property.
 
             If not editable, just display the value via plain().
@@ -1485,11 +1548,14 @@ class DateHTMLProperty(HTMLProperty):
         return DateHTMLProperty(self._client, self._classname, self._nodeid,
             self._prop, self._formname, ret)
 
-    def field(self, size=30, default=None, format=_marker):
+    def field(self, size=30, default=None, format=_marker, popcal=True):
         '''Render a form edit field for the property
 
         If not editable, just display the value via plain().
 
+        If "popcal" then include the Javascript calendar editor.
+        Default=yes.
+
         The format string is a standard python strftime format string.
         '''
         if not self.is_edit_ok():
@@ -1538,7 +1604,10 @@ class DateHTMLProperty(HTMLProperty):
                 value = value.pretty(format)
 
         value = cgi.escape(str(value), 1)
-        return self.input(name=self._formname, value=value, size=size)
+        s = self.input(name=self._formname, value=value, size=size)
+        if popcal:
+            s += self.popcal()
+        return s
 
     def reldate(self, pretty=1):
         ''' Render the interval between the date and now.
@@ -1589,7 +1658,7 @@ class DateHTMLProperty(HTMLProperty):
         return DateHTMLProperty(self._client, self._classname, self._nodeid,
             self._prop, self._formname, self._value, offset=offset)
 
-    def popcal(self, width=300, height=200, label= "(cal)",
+    def popcal(self, width=300, height=200, label="(cal)",
             form="itemSynopsis"):
         """Generate a link to a calendar pop-up window.
 
@@ -1630,7 +1699,7 @@ class IntervalHTMLProperty(HTMLProperty):
 
         return self._value.pretty()
 
-    def field(self, size = 30):
+    def field(self, size=30):
         ''' Render a form edit field for the property
 
             If not editable, just display the value via plain().
@@ -1711,8 +1780,9 @@ class LinkHTMLProperty(HTMLProperty):
                 value = self._value
             value = cgi.escape(str(value))
             value = '&quot;'.join(value.split('"'))
-        return '<input name="%s" value="%s" size="%s">'%(self._formname,
-            value, size)
+        return self.input(name=self._formname,
+                          value=value,
+                          size=size)
 
     def menu(self, size=None, height=None, showid=0, additional=[], value=None,
             sort_on=None, **conditions):
@@ -2084,6 +2154,41 @@ class HTMLRequest(HTMLInputMixin):
             args['@template'] = self.template
         return self.indexargs_url(url, args)
 
+    def _parse_sort(self, var, name):
+        ''' Parse sort/group options. Append to var
+        '''
+        fields = []
+        dirs = []
+        for special in '@:':
+            idx = 0
+            key = '%s%s%d'%(special, name, idx)
+            while key in self.form:
+                self.special_char = special
+                fields.append (self.form[key].value)
+                dirkey = '%s%sdir%d'%(special, name, idx)
+                if dirkey in self.form:
+                    dirs.append(self.form[dirkey].value)
+                else:
+                    dirs.append(None)
+                idx += 1
+                key = '%s%s%d'%(special, name, idx)
+            # backward compatible (and query) URL format
+            key = special + name
+            dirkey = key + 'dir'
+            if key in self.form and not fields:
+                fields = handleListCGIValue(self.form[key])
+                if dirkey in self.form:
+                    dirs.append(self.form[dirkey].value)
+            if fields: # only try other special char if nothing found
+                break
+        for f, d in map(None, fields, dirs):
+            if f.startswith('-'):
+                var.append(('-', f[1:]))
+            elif d:
+                var.append(('-', f))
+            else:
+                var.append(('+', f))
+
     def _post_init(self):
         ''' Set attributes based on self.form
         '''
@@ -2096,31 +2201,11 @@ class HTMLRequest(HTMLInputMixin):
                 break
         self.show = support.TruthDict(self.columns)
 
-        # sorting
-        self.sort = (None, None)
-        for name in ':sort @sort'.split():
-            if self.form.has_key(name):
-                self.special_char = name[0]
-                sort = self.form[name].value
-                if sort.startswith('-'):
-                    self.sort = ('-', sort[1:])
-                else:
-                    self.sort = ('+', sort)
-                if self.form.has_key(self.special_char+'sortdir'):
-                    self.sort = ('-', self.sort[1])
-
-        # grouping
-        self.group = (None, None)
-        for name in ':group @group'.split():
-            if self.form.has_key(name):
-                self.special_char = name[0]
-                group = self.form[name].value
-                if group.startswith('-'):
-                    self.group = ('-', group[1:])
-                else:
-                    self.group = ('+', group)
-                if self.form.has_key(self.special_char+'groupdir'):
-                    self.group = ('-', self.group[1])
+        # sorting and grouping
+        self.sort = []
+        self.group = []
+        self._parse_sort(self.sort, 'sort')
+        self._parse_sort(self.group, 'group')
 
         # filtering
         self.filter = []
@@ -2132,11 +2217,11 @@ class HTMLRequest(HTMLInputMixin):
         self.filterspec = {}
         db = self.client.db
         if self.classname is not None:
-            props = db.getclass(self.classname).getprops()
+            cls = db.getclass (self.classname)
             for name in self.filter:
                 if not self.form.has_key(name):
                     continue
-                prop = props[name]
+                prop = cls.get_transitive_prop (name)
                 fv = self.form[name]
                 if (isinstance(prop, hyperdb.Link) or
                         isinstance(prop, hyperdb.Multilink)):
@@ -2173,9 +2258,10 @@ class HTMLRequest(HTMLInputMixin):
                 self.startwith = int(self.form[name].value)
 
         # dispname
-        self.dispname = None
         if self.form.has_key('@dispname'):
             self.dispname = self.form['@dispname'].value
+        else:
+            self.dispname = None
 
     def updateFromURL(self, url):
         ''' Parse the URL for query args, and update my attributes using the
@@ -2245,18 +2331,22 @@ env: %(env)s
         s = self.input(type="hidden",name="%s",value="%s")
         if columns and self.columns:
             l.append(s%(sc+'columns', ','.join(self.columns)))
-        if sort and self.sort[1] is not None:
-            if self.sort[0] == '-':
-                val = '-'+self.sort[1]
-            else:
-                val = self.sort[1]
-            l.append(s%(sc+'sort', val))
-        if group and self.group[1] is not None:
-            if self.group[0] == '-':
-                val = '-'+self.group[1]
-            else:
-                val = self.group[1]
-            l.append(s%(sc+'group', val))
+        if sort:
+            val = []
+            for dir, attr in self.sort:
+                if dir == '-':
+                    val.append('-'+attr)
+                else:
+                    val.append(attr)
+            l.append(s%(sc+'sort', ','.join (val)))
+        if group:
+            val = []
+            for dir, attr in self.group:
+                if dir == '-':
+                    val.append('-'+attr)
+                else:
+                    val.append(attr)
+            l.append(s%(sc+'group', ','.join (val)))
         if filter and self.filter:
             l.append(s%(sc+'filter', ','.join(self.filter)))
         if self.classname and filterspec:
@@ -2291,18 +2381,22 @@ env: %(env)s
         # ok, now handle the specials we received in the request
         if self.columns and not specials.has_key('columns'):
             l.append(sc+'columns=%s'%(','.join(self.columns)))
-        if self.sort[1] is not None and not specials.has_key('sort'):
-            if self.sort[0] == '-':
-                val = '-'+self.sort[1]
-            else:
-                val = self.sort[1]
-            l.append(sc+'sort=%s'%val)
-        if self.group[1] is not None and not specials.has_key('group'):
-            if self.group[0] == '-':
-                val = '-'+self.group[1]
-            else:
-                val = self.group[1]
-            l.append(sc+'group=%s'%val)
+        if self.sort and not specials.has_key('sort'):
+            val = []
+            for dir, attr in self.sort:
+                if dir == '-':
+                    val.append('-'+attr)
+                else:
+                    val.append(attr)
+            l.append(sc+'sort=%s'%(','.join(val)))
+        if self.group and not specials.has_key('group'):
+            val = []
+            for dir, attr in self.group:
+                if dir == '-':
+                    val.append('-'+attr)
+                else:
+                    val.append(attr)
+            l.append(sc+'group=%s'%(','.join(val)))
         if self.filter and not specials.has_key('filter'):
             l.append(sc+'filter=%s'%(','.join(self.filter)))
         if self.search_text and not specials.has_key('search_text'):
@@ -2314,11 +2408,12 @@ env: %(env)s
 
         # finally, the remainder of the filter args in the request
         if self.classname and self.filterspec:
-            props = self.client.db.getclass(self.classname).getprops()
+            cls = self.client.db.getclass(self.classname)
             for k,v in self.filterspec.items():
                 if not args.has_key(k):
                     if type(v) == type([]):
-                        if isinstance(props[k], hyperdb.String):
+                        prop = cls.get_transitive_prop(k)
+                        if isinstance(prop, hyperdb.String):
                             l.append('%s=%s'%(k, '%20'.join([q(i) for i in v])))
                         else:
                             l.append('%s=%s'%(k, ','.join([q(i) for i in v])))
@@ -2433,16 +2528,23 @@ class Batch(ZTUtils.Batch):
         self.current_item = item
         return item
 
-    def propchanged(self, property):
-        ''' Detect if the property marked as being the group property
-            changed in the last iteration fetch
+    def propchanged(self, *properties):
+        ''' Detect if one of the properties marked as being a group
+            property changed in the last iteration fetch
         '''
         # we poke directly at the _value here since MissingValue can screw
         # us up and cause Nones to compare strangely
-        if (self.last_item is None or
-                self.last_item[property]._value !=
-                self.current_item[property]._value):
+        if self.last_item is None:
             return 1
+        for property in properties:
+            if property == 'id' or isinstance (self.last_item[property], list):
+                if (str(self.last_item[property]) !=
+                    str(self.current_item[property])):
+                    return 1
+            else:
+                if (self.last_item[property]._value !=
+                    self.current_item[property]._value):
+                    return 1
         return 0
 
     # override these 'cos we don't have access to acquisition
@@ -2476,7 +2578,7 @@ class TemplatingUtils:
 
     def html_quote(self, html):
         '''HTML-quote the supplied text.'''
-        return cgi.escape(url)
+        return cgi.escape(html)
 
     def __getattr__(self, name):
         '''Try the tracker's templating_utils.'''
