@@ -141,7 +141,14 @@ class Mailer:
 
         writer.lastpart()
 
-        self.smtp_send(to, message)
+        try:
+            self.smtp_send(to, message)
+        except MessageSendError:
+            # squash mail sending errors when bouncing mail
+            # TODO this *could* be better, as we could notify admin of the
+            # problem (even though the vast majority of bounce errors are
+            # because of spam)
+            pass
 
     def exception_message(self):
         '''Send a message to the admins with information about the latest
