@@ -535,19 +535,7 @@ class MailGW:
             # just inform the user that he is not authorized
             m = ['']
             m.append(str(value))
-            try:
-                self.mailer.bounce_message(message, [sendto[0][1]], m)
-            except MessageSendError, error:
-                # if the only reason the bounce failed is because of
-                # invalid addresses to bounce the message back to, then
-                # just discard the message - it's just not worth bothering
-                # with (most likely spam / otherwise forged)
-                invalid = True
-                for address, (code, reason) in error.keys():
-                    if code != 550:
-                        invalid = False
-                if not invalid:
-                    raise
+            self.mailer.bounce_message(message, [sendto[0][1]], m)
         except IgnoreMessage:
             # do not take any action
             # this exception is thrown when email should be ignored
@@ -624,18 +612,20 @@ Emails to Roundup trackers must include a Subject: line!
         # Re: "[issue1234] title of issue [status=resolved]"
         open, close = config['MAILGW_SUBJECT_SUFFIX_DELIMITERS']
         delim_open = re.escape(open)
+        if delim_open in '[(': delim_open = '\\' + delim_open
         delim_close = re.escape(close)
-        subject_re = re.compile(r'''
+        if delim_close in '[(': delim_close = '\\' + delim_close
+        subject_re = r'''
         (?P<refwd>\s*\W?\s*(fw|fwd|re|aw)\W\s*)*\s*   # Re:
         (?P<quote>")?                                 # Leading "
-        (\[(?P<classname>[^\d\s]+)                    # [issue..
+        (%s(?P<classname>[^\d\s]+)                    # [issue..
            (?P<nodeid>\d+)?                           # ..1234]
-         \])?\s*
+         %s)?\s*
         (?P<title>[^%s]+)?                             # issue title
         "?                                            # Trailing "
         (?P<argswhole>%s(?P<args>.+?)%s)?             # [prop=value]
-        '''%(delim_open, delim_open, delim_close),
-        re.IGNORECASE|re.VERBOSE)
+        '''%(delim_open, delim_close, delim_open, delim_open, delim_close)
+        subject_re = re.compile(subject_re, re.IGNORECASE|re.VERBOSE)
 
         # figure subject line parsing modes
         pfxmode = config['MAILGW_SUBJECT_PREFIX_PARSING']
@@ -716,6 +706,13 @@ Subject was: "%(subject)s"
         else:
             nodeid = m.group('nodeid')
 
+        # try in-reply-to to match the message if there's no nodeid
+        inreplyto = message.getheader('in-reply-to') or ''
+        if nodeid is None and inreplyto:
+            l = self.db.getclass('msg').stringFind(inreplyto=inreplyto)
+            if l:
+                nodeid = l[0]
+
         # title is optional too
         title = m.group('title')
         if title:
@@ -930,7 +927,6 @@ Subject was: "%(subject)s"
         # handle message-id and in-reply-to
         #
         messageid = message.getheader('message-id')
-        inreplyto = message.getheader('in-reply-to') or ''
         # generate a messageid if there isn't one
         if not messageid:
             messageid = "<%s.%s.%s%s@%s>"%(time.time(), random.random(),
