@@ -1,3 +1,4 @@
+from __future__ import nested_scopes
 #
 # Copyright (c) 2001 Bizar Software Pty Ltd (http://www.bizarsoftware.com.au/)
 # This module is free software, and you may redistribute it and/or modify
@@ -21,8 +22,6 @@
 """
 __docformat__ = 'restructuredtext'
 
-from __future__ import nested_scopes
-
 import re, os, smtplib, socket, time, random
 import cStringIO, base64, quopri, mimetypes
 
@@ -72,13 +71,18 @@ class Database:
         If no such property exists return 0
         """
         userid = self.getuid()
+        timezone = None
         try:
-            timezone = int(self.user.get(userid, 'timezone'))
-        except (KeyError, ValueError, TypeError):
-            # If there is no class 'user' or current user doesn't have timezone
-            # property or that property is not numeric assume he/she lives in
-            # Greenwich :)
-            timezone = getattr(self.config, 'DEFAULT_TIMEZONE', 0)
+            tz = self.user.get(userid, 'timezone')
+            date.get_timezone(tz)
+            timezone = tz
+        except KeyError:
+            pass
+        # If there is no class 'user' or current user doesn't have timezone
+        # property or that property is not set assume he/she lives in
+        # the timezone set in the tracker config.
+        if timezone is None:
+            timezone = self.config['TIMEZONE']
         return timezone
 
     def confirm_registration(self, otk):
