@@ -69,7 +69,8 @@ def setupSchema(db, create, module):
     priority = module.Class(db, "priority", name=String(), order=String())
     priority.setkey("name")
     user = module.Class(db, "user", username=String(), password=Password(),
-        assignable=Boolean(), age=Number(), roles=String())
+        assignable=Boolean(), age=Number(), roles=String(),
+        supervisor=Link('user'))
     user.setkey("username")
     file = module.FileClass(db, "file", name=String(), type=String(),
         comment=String(indexme="yes"), fooz=Password())
@@ -80,7 +81,7 @@ def setupSchema(db, create, module):
         priority=Link('priority'))
     stuff = module.Class(db, "stuff", stuff=String())
     session = module.Class(db, 'session', title=String())
-    msg = module.FileClass(db, "msg",
+    msg = module.FileClass(db, "msg", date=Date(),
                            author=Link("user", do_journal='no'))
     session.disableJournalling()
     db.post_init()
@@ -927,6 +928,8 @@ class DBTest(MyTestCase):
                 {'username': 'blorp', 'age': 2}):
             self.db.user.create(**user)
         iss = self.db.issue
+        file_content = ''.join([chr(i) for i in range(255)])
+        f = self.db.file.create(content=file_content)
         for issue in (
                 {'title': 'issue one', 'status': '2', 'assignedto': '1',
                     'foo': date.Interval('1:10'), 'priority': '3',
@@ -938,10 +941,9 @@ class DBTest(MyTestCase):
                     'nosy': ['1','2'], 'deadline': date.Date('2003-02-18')},
                 {'title': 'non four', 'status': '3',
                     'foo': date.Interval('0:10'), 'priority': '2',
-                    'nosy': ['1'], 'deadline': date.Date('2004-03-08')}):
+                    'nosy': ['1','2','3'], 'deadline': date.Date('2004-03-08'),
+                    'files': [f]}):
             self.db.issue.create(**issue)
-        file_content = ''.join([chr(i) for i in range(255)])
-        self.db.file.create(content=file_content)
         self.db.commit()
         return self.assertEqual, self.db.issue.filter
 
@@ -949,7 +951,7 @@ class DBTest(MyTestCase):
         ae, filt = self.filteringSetup()
         ae(filt(None, {'id': '1'}, ('+','id'), (None,None)), ['1'])
         ae(filt(None, {'id': '2'}, ('+','id'), (None,None)), ['2'])
-        ae(filt(None, {'id': '10'}, ('+','id'), (None,None)), [])
+        ae(filt(None, {'id': '100'}, ('+','id'), (None,None)), [])
 
     def testFilteringNumber(self):
         self.filteringSetup()
@@ -976,11 +978,21 @@ class DBTest(MyTestCase):
         ae(filt(None, {'status': '1'}, ('+','id'), (None,None)), ['2','3'])
         ae(filt(None, {'assignedto': '-1'}, ('+','id'), (None,None)), ['3','4'])
         ae(filt(None, {'assignedto': None}, ('+','id'), (None,None)), ['3','4'])
+        ae(filt(None, {'assignedto': [None]}, ('+','id'), (None,None)),
+            ['3','4'])
         ae(filt(None, {'assignedto': ['-1', None]}, ('+','id'), (None,None)),
             ['3','4'])
         ae(filt(None, {'assignedto': ['1', None]}, ('+','id'), (None,None)),
             ['1', '3','4'])
 
+    def testFilteringMultilinkAndGroup(self):
+        """testFilteringMultilinkAndGroup:
+        See roundup Bug 1541128: apparently grouping by something and
+        searching a Multilink failed with MySQL 5.0
+        """
+        ae, filt = self.filteringSetup()
+        ae(filt(None, {'files': '1'}, ('-','activity'), ('+','status')), ['4'])
+
     def testFilteringRetired(self):
         ae, filt = self.filteringSetup()
         self.db.issue.retire('2')
@@ -988,7 +1000,7 @@ class DBTest(MyTestCase):
 
     def testFilteringMultilink(self):
         ae, filt = self.filteringSetup()
-        ae(filt(None, {'nosy': '2'}, ('+','id'), (None,None)), ['3'])
+        ae(filt(None, {'nosy': '3'}, ('+','id'), (None,None)), ['4'])
         ae(filt(None, {'nosy': '-1'}, ('+','id'), (None,None)), ['1', '2'])
         ae(filt(None, {'nosy': ['1','2']}, ('+', 'status'),
             ('-', 'deadline')), ['4', '3'])
@@ -1045,23 +1057,75 @@ class DBTest(MyTestCase):
         # descending should sort 1d, 1:10, None
         ae(filt(None, {}, ('-','foo'), (None,None)), ['2', '1', '4', '3'])
 
+    def testFilteringStringSort(self):
+        # 1: 'issue one'
+        # 2: 'issue two'
+        # 3: 'issue three'
+        # 4: 'non four'
+        ae, filt = self.filteringSetup()
+        ae(filt(None, {}, ('+','title')), ['1', '3', '2', '4'])
+        ae(filt(None, {}, ('-','title')), ['4', '2', '3', '1'])
+        # Test string case: For now allow both, w/wo case matching.
+        # 1: 'issue one'
+        # 2: 'issue two'
+        # 3: 'Issue three'
+        # 4: 'non four'
+        self.db.issue.set('3', title='Issue three')
+        ae(filt(None, {}, ('+','title')), ['1', '3', '2', '4'])
+        ae(filt(None, {}, ('-','title')), ['4', '2', '3', '1'])
+        # Obscure bug in anydbm backend trying to convert to number
+        # 1: '1st issue'
+        # 2: '2'
+        # 3: 'Issue three'
+        # 4: 'non four'
+        self.db.issue.set('1', title='1st issue')
+        self.db.issue.set('2', title='2')
+        ae(filt(None, {}, ('+','title')), ['1', '2', '3', '4'])
+        ae(filt(None, {}, ('-','title')), ['4', '3', '2', '1'])
+
     def testFilteringMultilinkSort(self):
-        # 1: []
-        # 2: []
-        # 3: ['1','2']
-        # 4: ['1']
+        # 1: []                 Reverse:  1: []
+        # 2: []                           2: []              
+        # 3: ['admin','fred']             3: ['fred','admin']       
+        # 4: ['admin','bleep','fred']     4: ['fred','bleep','admin']
+        # Note the sort order for the multilink doen't change when
+        # reversing the sort direction due to the re-sorting of the
+        # multilink!
         ae, filt = self.filteringSetup()
         ae(filt(None, {}, ('+','nosy'), (None,None)), ['1', '2', '4', '3'])
-        ae(filt(None, {}, ('-','nosy'), (None,None)), ['3', '4', '1', '2'])
+        ae(filt(None, {}, ('-','nosy'), (None,None)), ['4', '3', '1', '2'])
+
+    def testFilteringMultilinkSortGroup(self):
+        # 1: status: 2 "in-progress" nosy: []
+        # 2: status: 1 "unread"      nosy: []
+        # 3: status: 1 "unread"      nosy: ['admin','fred']
+        # 4: status: 3 "testing"     nosy: ['admin','bleep','fred']
+        ae, filt = self.filteringSetup()
+        ae(filt(None, {}, ('+','nosy'), ('+','status')), ['1', '4', '2', '3'])
+        ae(filt(None, {}, ('-','nosy'), ('+','status')), ['1', '4', '3', '2'])
+        ae(filt(None, {}, ('+','nosy'), ('-','status')), ['2', '3', '4', '1'])
+        ae(filt(None, {}, ('-','nosy'), ('-','status')), ['3', '2', '4', '1'])
+        ae(filt(None, {}, ('+','status'), ('+','nosy')), ['1', '2', '4', '3'])
+        ae(filt(None, {}, ('-','status'), ('+','nosy')), ['2', '1', '4', '3'])
+        ae(filt(None, {}, ('+','status'), ('-','nosy')), ['4', '3', '1', '2'])
+        ae(filt(None, {}, ('-','status'), ('-','nosy')), ['4', '3', '2', '1'])
 
     def testFilteringLinkSortGroup(self):
-        # 1: status: 2, priority: 3
-        # 2: status: 1, priority: 3
-        # 3: status: 1, priority: 2,
-        # 4: status: 3, priority: 2,
+        # 1: status: 2 -> 'i', priority: 3 -> 1
+        # 2: status: 1 -> 'u', priority: 3 -> 1
+        # 3: status: 1 -> 'u', priority: 2 -> 3
+        # 4: status: 3 -> 't', priority: 2 -> 3
         ae, filt = self.filteringSetup()
         ae(filt(None, {}, ('+','status'), ('+','priority')),
             ['1', '2', '4', '3'])
+        ae(filt(None, {'priority':'2'}, ('+','status'), ('+','priority')),
+            ['4', '3'])
+        ae(filt(None, {'priority.order':'3'}, ('+','status'), ('+','priority')),
+            ['4', '3'])
+        ae(filt(None, {'priority':['2','3']}, ('+','priority'), ('+','status')),
+            ['1', '4', '2', '3'])
+        ae(filt(None, {}, ('+','priority'), ('+','status')),
+            ['1', '4', '2', '3'])
 
     def testFilteringDateSort(self):
         # '1': '2003-02-16.22:50'
@@ -1092,6 +1156,255 @@ class DBTest(MyTestCase):
         ae(filt(None, {}, ('-','deadline'), ('-','priority')),
             ['4', '3', '1', '2'])
 
+    def filteringSetupTransitiveSearch(self):
+        u_m = {}
+        k = 30
+        for user in (
+                {'username': 'ceo', 'age': 129},
+                {'username': 'grouplead1', 'age': 29, 'supervisor': '3'},
+                {'username': 'grouplead2', 'age': 29, 'supervisor': '3'},
+                {'username': 'worker1', 'age': 25, 'supervisor' : '4'},
+                {'username': 'worker2', 'age': 24, 'supervisor' : '4'},
+                {'username': 'worker3', 'age': 23, 'supervisor' : '5'},
+                {'username': 'worker4', 'age': 22, 'supervisor' : '5'},
+                {'username': 'worker5', 'age': 21, 'supervisor' : '5'}):
+            u = self.db.user.create(**user)
+            u_m [u] = self.db.msg.create(author = u, content = ' '
+                , date = date.Date ('2006-01-%s' % k))
+            k -= 1
+        iss = self.db.issue
+        for issue in (
+                {'title': 'ts1', 'status': '2', 'assignedto': '6',
+                    'priority': '3', 'messages' : [u_m ['6']], 'nosy' : ['4']},
+                {'title': 'ts2', 'status': '1', 'assignedto': '6',
+                    'priority': '3', 'messages' : [u_m ['6']], 'nosy' : ['5']},
+                {'title': 'ts4', 'status': '2', 'assignedto': '7',
+                    'priority': '3', 'messages' : [u_m ['7']]},
+                {'title': 'ts5', 'status': '1', 'assignedto': '8',
+                    'priority': '3', 'messages' : [u_m ['8']]},
+                {'title': 'ts6', 'status': '2', 'assignedto': '9',
+                    'priority': '3', 'messages' : [u_m ['9']]},
+                {'title': 'ts7', 'status': '1', 'assignedto': '10',
+                    'priority': '3', 'messages' : [u_m ['10']]},
+                {'title': 'ts8', 'status': '2', 'assignedto': '10',
+                    'priority': '3', 'messages' : [u_m ['10']]},
+                {'title': 'ts9', 'status': '1', 'assignedto': '10',
+                    'priority': '3', 'messages' : [u_m ['10'], u_m ['9']]}):
+            self.db.issue.create(**issue)
+        return self.assertEqual, self.db.issue.filter
+
+    def testFilteringTransitiveLinkUser(self):
+        ae, filt = self.filteringSetupTransitiveSearch()
+        ufilt = self.db.user.filter
+        ae(ufilt(None, {'supervisor.username': 'ceo'}, ('+','username')),
+            ['4', '5'])
+        ae(ufilt(None, {'supervisor.supervisor.username': 'ceo'},
+            ('+','username')), ['6', '7', '8', '9', '10'])
+        ae(ufilt(None, {'supervisor.supervisor': '3'}, ('+','username')),
+            ['6', '7', '8', '9', '10'])
+        ae(ufilt(None, {'supervisor.supervisor.id': '3'}, ('+','username')),
+            ['6', '7', '8', '9', '10'])
+        ae(ufilt(None, {'supervisor.username': 'grouplead1'}, ('+','username')),
+            ['6', '7'])
+        ae(ufilt(None, {'supervisor.username': 'grouplead2'}, ('+','username')),
+            ['8', '9', '10'])
+        ae(ufilt(None, {'supervisor.username': 'grouplead2',
+            'supervisor.supervisor.username': 'ceo'}, ('+','username')),
+            ['8', '9', '10'])
+        ae(ufilt(None, {'supervisor.supervisor': '3', 'supervisor': '4'},
+            ('+','username')), ['6', '7'])
+
+    def testFilteringTransitiveLinkSort(self):
+        ae, filt = self.filteringSetupTransitiveSearch()
+        ufilt = self.db.user.filter
+        # Need to make ceo his own (and first two users') supervisor,
+        # otherwise we will depend on sorting order of NULL values.
+        # Leave that to a separate test.
+        self.db.user.set('1', supervisor = '3')
+        self.db.user.set('2', supervisor = '3')
+        self.db.user.set('3', supervisor = '3')
+        ae(ufilt(None, {'supervisor':'3'}, []), ['1', '2', '3', '4', '5'])
+        ae(ufilt(None, {}, [('+','supervisor.supervisor.supervisor'),
+            ('+','supervisor.supervisor'), ('+','supervisor'),
+            ('+','username')]),
+            ['1', '3', '2', '4', '5', '6', '7', '8', '9', '10'])
+        ae(ufilt(None, {}, [('+','supervisor.supervisor.supervisor'),
+            ('-','supervisor.supervisor'), ('-','supervisor'),
+            ('+','username')]),
+            ['8', '9', '10', '6', '7', '1', '3', '2', '4', '5'])
+        ae(filt(None, {}, [('+','assignedto.supervisor.supervisor.supervisor'),
+            ('+','assignedto.supervisor.supervisor'),
+            ('+','assignedto.supervisor'), ('+','assignedto')]),
+            ['1', '2', '3', '4', '5', '6', '7', '8'])
+        ae(filt(None, {}, [('+','assignedto.supervisor.supervisor.supervisor'),
+            ('+','assignedto.supervisor.supervisor'),
+            ('-','assignedto.supervisor'), ('+','assignedto')]),
+            ['4', '5', '6', '7', '8', '1', '2', '3'])
+        ae(filt(None, {}, [('+','assignedto.supervisor.supervisor.supervisor'),
+            ('+','assignedto.supervisor.supervisor'),
+            ('+','assignedto.supervisor'), ('+','assignedto'),
+            ('-','status')]),
+            ['2', '1', '3', '4', '5', '6', '8', '7'])
+        ae(filt(None, {}, [('+','assignedto.supervisor.supervisor.supervisor'),
+            ('+','assignedto.supervisor.supervisor'),
+            ('+','assignedto.supervisor'), ('+','assignedto'),
+            ('+','status')]),
+            ['1', '2', '3', '4', '5', '7', '6', '8'])
+        ae(filt(None, {}, [('+','assignedto.supervisor.supervisor.supervisor'),
+            ('+','assignedto.supervisor.supervisor'),
+            ('-','assignedto.supervisor'), ('+','assignedto'), ('+','status')]),
+            ['4', '5', '7', '6', '8', '1', '2', '3'])
+        ae(filt(None, {'assignedto':['6','7','8','9','10']},
+            [('+','assignedto.supervisor.supervisor.supervisor'),
+            ('+','assignedto.supervisor.supervisor'),
+            ('-','assignedto.supervisor'), ('+','assignedto'), ('+','status')]),
+            ['4', '5', '7', '6', '8', '1', '2', '3'])
+        ae(filt(None, {'assignedto':['6','7','8','9']},
+            [('+','assignedto.supervisor.supervisor.supervisor'),
+            ('+','assignedto.supervisor.supervisor'),
+            ('-','assignedto.supervisor'), ('+','assignedto'), ('+','status')]),
+            ['4', '5', '1', '2', '3'])
+
+    def testFilteringTransitiveLinkSortNull(self):
+        """Check sorting of NULL values"""
+        ae, filt = self.filteringSetupTransitiveSearch()
+        ufilt = self.db.user.filter
+        ae(ufilt(None, {}, [('+','supervisor.supervisor.supervisor'),
+            ('+','supervisor.supervisor'), ('+','supervisor'),
+            ('+','username')]),
+            ['1', '3', '2', '4', '5', '6', '7', '8', '9', '10'])
+        ae(ufilt(None, {}, [('+','supervisor.supervisor.supervisor'),
+            ('-','supervisor.supervisor'), ('-','supervisor'),
+            ('+','username')]),
+            ['8', '9', '10', '6', '7', '4', '5', '1', '3', '2'])
+        ae(filt(None, {}, [('+','assignedto.supervisor.supervisor.supervisor'),
+            ('+','assignedto.supervisor.supervisor'),
+            ('+','assignedto.supervisor'), ('+','assignedto')]),
+            ['1', '2', '3', '4', '5', '6', '7', '8'])
+        ae(filt(None, {}, [('+','assignedto.supervisor.supervisor.supervisor'),
+            ('+','assignedto.supervisor.supervisor'),
+            ('-','assignedto.supervisor'), ('+','assignedto')]),
+            ['4', '5', '6', '7', '8', '1', '2', '3'])
+
+    def testFilteringTransitiveLinkIssue(self):
+        ae, filt = self.filteringSetupTransitiveSearch()
+        ae(filt(None, {'assignedto.supervisor.username': 'grouplead1'},
+            ('+','id')), ['1', '2', '3'])
+        ae(filt(None, {'assignedto.supervisor.username': 'grouplead2'},
+            ('+','id')), ['4', '5', '6', '7', '8'])
+        ae(filt(None, {'assignedto.supervisor.username': 'grouplead2',
+                       'status': '1'}, ('+','id')), ['4', '6', '8'])
+        ae(filt(None, {'assignedto.supervisor.username': 'grouplead2',
+                       'status': '2'}, ('+','id')), ['5', '7'])
+        ae(filt(None, {'assignedto.supervisor.username': ['grouplead2'],
+                       'status': '2'}, ('+','id')), ['5', '7'])
+        ae(filt(None, {'assignedto.supervisor': ['4', '5'], 'status': '2'},
+            ('+','id')), ['1', '3', '5', '7'])
+
+    def testFilteringTransitiveMultilink(self):
+        ae, filt = self.filteringSetupTransitiveSearch()
+        ae(filt(None, {'messages.author.username': 'grouplead1'},
+            ('+','id')), [])
+        ae(filt(None, {'messages.author': '6'},
+            ('+','id')), ['1', '2'])
+        ae(filt(None, {'messages.author.id': '6'},
+            ('+','id')), ['1', '2'])
+        ae(filt(None, {'messages.author.username': 'worker1'},
+            ('+','id')), ['1', '2'])
+        ae(filt(None, {'messages.author': '10'},
+            ('+','id')), ['6', '7', '8'])
+        ae(filt(None, {'messages.author': '9'},
+            ('+','id')), ['5', '8'])
+        ae(filt(None, {'messages.author': ['9', '10']},
+            ('+','id')), ['5', '6', '7', '8'])
+        ae(filt(None, {'messages.author': ['8', '9']},
+            ('+','id')), ['4', '5', '8'])
+        ae(filt(None, {'messages.author': ['8', '9'], 'status' : '1'},
+            ('+','id')), ['4', '8'])
+        ae(filt(None, {'messages.author': ['8', '9'], 'status' : '2'},
+            ('+','id')), ['5'])
+        ae(filt(None, {'messages.author': ['8', '9', '10'],
+            'messages.date': '2006-01-22.21:00;2006-01-23'}, ('+','id')),
+            ['6', '7', '8'])
+        ae(filt(None, {'nosy.supervisor.username': 'ceo'},
+            ('+','id')), ['1', '2'])
+        ae(filt(None, {'messages.author': ['6', '9']},
+            ('+','id')), ['1', '2', '5', '8'])
+        ae(filt(None, {'messages': ['5', '7']},
+            ('+','id')), ['3', '5', '8'])
+        ae(filt(None, {'messages.author': ['6', '9'], 'messages': ['5', '7']},
+            ('+','id')), ['5', '8'])
+
+    def testFilteringTransitiveMultilinkSort(self):
+        ae, filt = self.filteringSetupTransitiveSearch()
+        ae(filt(None, {}, [('+','messages.author')]),
+            ['1', '2', '3', '4', '5', '8', '6', '7'])
+        ae(filt(None, {}, [('-','messages.author')]),
+            ['8', '6', '7', '5', '4', '3', '1', '2'])
+        ae(filt(None, {}, [('+','messages.date')]),
+            ['6', '7', '8', '5', '4', '3', '1', '2'])
+        ae(filt(None, {}, [('-','messages.date')]),
+            ['1', '2', '3', '4', '8', '5', '6', '7'])
+        ae(filt(None, {}, [('+','messages.author'),('+','messages.date')]),
+            ['1', '2', '3', '4', '5', '8', '6', '7'])
+        ae(filt(None, {}, [('-','messages.author'),('+','messages.date')]),
+            ['8', '6', '7', '5', '4', '3', '1', '2'])
+        ae(filt(None, {}, [('+','messages.author'),('-','messages.date')]),
+            ['1', '2', '3', '4', '5', '8', '6', '7'])
+        ae(filt(None, {}, [('-','messages.author'),('-','messages.date')]),
+            ['8', '6', '7', '5', '4', '3', '1', '2'])
+        ae(filt(None, {}, [('+','messages.author'),('+','assignedto')]),
+            ['1', '2', '3', '4', '5', '8', '6', '7'])
+        ae(filt(None, {}, [('+','messages.author'),
+            ('-','assignedto.supervisor'),('-','assignedto')]),
+            ['1', '2', '3', '4', '5', '8', '6', '7'])
+        ae(filt(None, {},
+            [('+','messages.author.supervisor.supervisor.supervisor'),
+            ('+','messages.author.supervisor.supervisor'),
+            ('+','messages.author.supervisor'), ('+','messages.author')]),
+            ['1', '2', '3', '4', '5', '6', '7', '8'])
+        self.db.user.setorderprop('age')
+        self.db.msg.setorderprop('date')
+        ae(filt(None, {}, [('+','messages'), ('+','messages.author')]),
+            ['6', '7', '8', '5', '4', '3', '1', '2'])
+        ae(filt(None, {}, [('+','messages.author'), ('+','messages')]),
+            ['6', '7', '8', '5', '4', '3', '1', '2'])
+        self.db.msg.setorderprop('author')
+        # Orderprop is a Link/Multilink:
+        # messages are sorted by orderprop().labelprop(), i.e. by
+        # author.username, *not* by author.orderprop() (author.age)!
+        ae(filt(None, {}, [('+','messages')]),
+            ['1', '2', '3', '4', '5', '8', '6', '7'])
+        ae(filt(None, {}, [('+','messages.author'), ('+','messages')]),
+            ['6', '7', '8', '5', '4', '3', '1', '2'])
+        # The following will sort by 
+        # author.supervisor.username and then by
+        # author.username
+        # I've resited the tempation to implement recursive orderprop
+        # here: There could even be loops if several classes specify a
+        # Link or Multilink as the orderprop...
+        # msg: 4: worker1 (id  5) : grouplead1 (id 4) ceo (id 3)
+        # msg: 5: worker2 (id  7) : grouplead1 (id 4) ceo (id 3)
+        # msg: 6: worker3 (id  8) : grouplead2 (id 5) ceo (id 3)
+        # msg: 7: worker4 (id  9) : grouplead2 (id 5) ceo (id 3)
+        # msg: 8: worker5 (id 10) : grouplead2 (id 5) ceo (id 3)
+        # issue 1: messages 4   sortkey:[[grouplead1], [worker1], 1]
+        # issue 2: messages 4   sortkey:[[grouplead1], [worker1], 2]
+        # issue 3: messages 5   sortkey:[[grouplead1], [worker2], 3]
+        # issue 4: messages 6   sortkey:[[grouplead2], [worker3], 4]
+        # issue 5: messages 7   sortkey:[[grouplead2], [worker4], 5]
+        # issue 6: messages 8   sortkey:[[grouplead2], [worker5], 6]
+        # issue 7: messages 8   sortkey:[[grouplead2], [worker5], 7]
+        # issue 8: messages 7,8 sortkey:[[grouplead2, grouplead2], ...]
+        self.db.user.setorderprop('supervisor')
+        ae(filt(None, {}, [('+','messages.author'), ('-','messages')]),
+            ['3', '1', '2', '6', '7', '5', '4', '8'])
+
+    def testFilteringSortId(self):
+        ae, filt = self.filteringSetupTransitiveSearch()
+        ae(self.db.user.filter(None, {}, ('+','id')),
+            ['1', '2', '3', '4', '5', '6', '7', '8', '9', '10'])
+
 # XXX add sorting tests for other types
 # XXX test auditors and reactors
 
