@@ -152,6 +152,10 @@ class FormTestCase(unittest.TestCase):
             {':required': 'status', 'status':''}, 'issue')
         self.assertRaises(FormError, self.parseForm,
             {':required': 'nosy', 'nosy':''}, 'issue')
+        self.assertRaises(FormError, self.parseForm,
+            {':required': 'msg-1@content', 'msg-1@content':''}, 'issue')
+        self.assertRaises(FormError, self.parseForm,
+            {':required': 'msg-1@content'}, 'issue')
 
     #
     # Nonexistant edit
