@@ -22,7 +22,11 @@
 
 import os, unittest, shutil
 
-from roundup.backends.indexer_dbm import Indexer
+class db:
+    class config(dict):
+        DATABASE = 'test-index'
+    config = config()
+    config[('main', 'indexer_stopwords')] = []
 
 class IndexerTest(unittest.TestCase):
     def setUp(self):
@@ -30,27 +34,56 @@ class IndexerTest(unittest.TestCase):
             shutil.rmtree('test-index')
         os.mkdir('test-index')
         os.mkdir('test-index/files')
-        self.dex = Indexer('test-index')
+        from roundup.backends.indexer_dbm import Indexer
+        self.dex = Indexer(db)
         self.dex.load_index()
 
     def test_basics(self):
-        self.dex.add_text('testing1', 'a the hello world')
-        self.assertEqual(self.dex.words, {'HELLO': {1: 1}, 'WORLD': {1: 1}})
-        self.dex.add_text('testing2', 'blah blah the world')
-        self.assertEqual(self.dex.words, {'BLAH': {2: 2}, 'HELLO': {1: 1},
-            'WORLD': {2: 1, 1: 1}})
-        self.assertEqual(self.dex.find(['world']), {2: 'testing2',
-            1: 'testing1'})
-        self.assertEqual(self.dex.find(['blah']), {2: 'testing2'})
-        self.assertEqual(self.dex.find(['blah', 'hello']), {})
-        self.dex.save_index()
+        self.dex.add_text(('test', '1', 'foo'), 'a the hello world')
+        self.dex.add_text(('test', '2', 'foo'), 'blah blah the world')
+        self.assertEqual(self.dex.find(['world']), [('test', '1', 'foo'),
+                                                    ('test', '2', 'foo')])
+        self.assertEqual(self.dex.find(['blah']), [('test', '2', 'foo')])
+        self.assertEqual(self.dex.find(['blah', 'hello']), [])
 
+    def test_change(self):
+        self.dex.add_text(('test', '1', 'foo'), 'a the hello world')
+        self.dex.add_text(('test', '2', 'foo'), 'blah blah the world')
+        self.assertEqual(self.dex.find(['world']), [('test', '1', 'foo'),
+                                                    ('test', '2', 'foo')])
+        self.dex.add_text(('test', '1', 'foo'), 'a the hello')
+        self.assertEqual(self.dex.find(['world']), [('test', '2', 'foo')])
+
+    def test_clear(self):
+        self.dex.add_text(('test', '1', 'foo'), 'a the hello world')
+        self.dex.add_text(('test', '2', 'foo'), 'blah blah the world')
+        self.assertEqual(self.dex.find(['world']), [('test', '1', 'foo'),
+                                                    ('test', '2', 'foo')])
+        self.dex.add_text(('test', '1', 'foo'), '')
+        self.assertEqual(self.dex.find(['world']), [('test', '2', 'foo')])
+
+    def tearDown(self):
+        shutil.rmtree('test-index')
+
+class XapianIndexerTest(IndexerTest):
+    def setUp(self):
+        if os.path.exists('test-index'):
+            shutil.rmtree('test-index')
+        os.mkdir('test-index')
+        from roundup.backends.indexer_xapian import Indexer
+        self.dex = Indexer(db)
     def tearDown(self):
         shutil.rmtree('test-index')
 
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(IndexerTest))
+    try:
+        import xapian
+        suite.addTest(unittest.makeSuite(XapianIndexerTest))
+    except ImportError:
+        print "Skipping Xapian indexer tests"
+        pass
     return suite
 
 if __name__ == '__main__':
