@@ -368,7 +368,6 @@ _______________________________________________________________________
 From: richard <richard@test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
-In-Reply-To: <dummy_test_message_id>
 Subject: Re: Testing... [assignedto=mary; nosy=+john]
 
 This is a followup
@@ -412,13 +411,12 @@ _______________________________________________________________________
 From: richard <richard@test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
-In-Reply-To: <dummy_test_message_id>
 Subject: Re: Testing...
 
 This is a followup
 '''), nodeid)
 
-    def testFollowupTitleMatchNever(self):
+    def testFollowupTitleMatchNeverInterval(self):
         nodeid = self.doNewIssue()
         # force failure of the interval
         time.sleep(2)
@@ -428,7 +426,6 @@ This is a followup
 From: richard <richard@test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
-In-Reply-To: <dummy_test_message_id>
 Subject: Re: Testing...
 
 This is a followup
@@ -440,7 +437,6 @@ This is a followup
 From: richard <richard@test>
 To: issue_tracker@your.tracker.email.domain.example
 Message-Id: <followup_dummy_id>
-In-Reply-To: <dummy_test_message_id>
 Subject: Re: Testing...
 
 This is a followup
@@ -1257,6 +1253,21 @@ Message-Id: <dummy_test_message_id>
         self.assertEqual(self.db.issue.get(nodeid, 'title'), 'testing')
         self.assertEqual(self.db.issue.get(nodeid, 'assignedto'), self.mary_id)
 
+    def testPrefixDelimiters(self):
+        self.instance.config.MAILGW_SUBJECT_SUFFIX_DELIMITERS = '{}'
+        self.db.keyword.create(name='Foo')
+        self._handle_mail('''Content-Type: text/plain;
+  charset="iso-8859-1"
+From: richard <richard@test>
+To: issue_tracker@your.tracker.email.domain.example
+Message-Id: <followup_dummy_id>
+In-Reply-To: <dummy_test_message_id>
+Subject: {keyword1} Testing... {name=Bar}
+
+''')
+        assert not os.path.exists(SENDMAILDEBUG)
+        self.assertEqual(self.db.keyword.get('1', 'name'), 'Bar')
+
     def testCommandDelimitersIgnore(self):
         self.instance.config.MAILGW_SUBJECT_SUFFIX_DELIMITERS = '{}'
         nodeid = self._handle_mail('''Content-Type: text/plain;
