@@ -167,6 +167,11 @@ blah blah blah signature
 userfoo@foo.com
 ''')
 
+    def testAllQuoted(self):
+        s = '\nissue_tracker@foo.com wrote:\n> testing\n'
+        summary, content = parseContent(s, 0, 1)
+        self.assertEqual(summary, '')
+        self.assertEqual(content, s)
 
     def testSimple(self):
         s = '''testing'''
@@ -214,8 +219,21 @@ userfoo@foo.com
         self.assertEqual(summary, 'This is a long sentence that would '
             'normally\nbe split.')
 
-def suite():
-   return unittest.makeSuite(MailsplitTestCase, 'test')
+    def testKeepMultipleHyphens(self):
+        body = '''Testing, testing.
 
+----
+Testing, testing.'''
+        summary, content = parseContent(body, 1, 0)
+        self.assertEqual(body, content)
+
+def test_suite():
+    suite = unittest.TestSuite()
+    suite.addTest(unittest.makeSuite(MailsplitTestCase))
+    return suite
+
+if __name__ == '__main__':
+    runner = unittest.TextTestRunner()
+    unittest.main(testRunner=runner)
 
 # vim: set filetype=python ts=4 sw=4 et si
